//////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2018 Leszek Koltunski                                                          //
//                                                                                          //
// This file is part of Distorted.                                                          //
//                                                                                          //
// Distorted is free software: you can redistribute it and/or modify                        //
// it under the terms of the GNU General Public License as published by                     //
// the Free Software Foundation, either version 2 of the License, or                        //
// (at your option) any later version.                                                      //
//                                                                                          //
// Distorted is distributed in the hope that it will be useful,                             //
// but WITHOUT ANY WARRANTY; without even the implied warranty of                           //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            //
// GNU General Public License for more details.                                             //
//                                                                                          //
// You should have received a copy of the GNU General Public License                        //
// along with Distorted.  If not, see <http://www.gnu.org/licenses/>.                       //
//////////////////////////////////////////////////////////////////////////////////////////////

precision highp float;
precision highp int;

out vec4 fragColor;           // The output color
in vec2 v_TexCoordinate;      // Interpolated texture coordinate per fragment.
in vec2 v_Pixel;              // location of the current fragment, in pixels

uniform sampler2D u_DepthTexture;

//////////////////////////////////////////////////////////////////////////////////////////////
// per-pixel linked list. Order Independent Transparency.

uniform uvec2 u_Size;

layout (std430,binding=1) buffer linkedlist  // first (u_Size.x*u_Size.y) uints - head pointers,
  {                                          // one for each pixel in the Output rectangle.
  uint u_Records[];                          //
  };                                         // Next 3*u_numRecords uints - actual linked list, i.e.
                                             // triplets of (pointer,depth,rgba).

//////////////////////////////////////////////////////////////////////////////////////////////
// Pass3 of the OIT algorithm - traverse the per-pixel LinkedList and cut occluded fragments.

void main()                    		
  {
  uint prev = uint(v_Pixel.x) + uint(v_Pixel.y) * u_Size.x;
  uint curr = u_Records[prev];

  if (curr != 0u)
    {
    const float S= 2147483647.0;
    float depth = texture(u_DepthTexture, v_TexCoordinate).r;
    uint texdepth = uint(S*(1.0-depth)/2.0);

    while( curr != 0u )
      {
      if( u_Records[curr+1u] <= texdepth )
        {
        u_Records[prev] = 0u;
        break;
        }

      prev = curr;
      curr = u_Records[curr];
      }
    }
  else discard;
  }